@@ -32,6 +32,11 @@ use alloc::string::String;
 #[cfg(not(feature = "std"))]
 use alloc::vec::Vec;
 
+#[cfg(not(feature = "std"))]
+use alloc::str::FromStr;
+#[cfg(feature = "std")]
+use std::str::FromStr;
+
 /// BLSValue generalizes a value that may have an inline comment
 #[derive(Debug, PartialEq)]
 pub enum BLSValue {
@@ -42,7 +47,7 @@ pub enum BLSValue {
 }
 
 /// This enum is used as an argument in the ```BLSEntry::set()``` method.
-/// Some times like ```initrd``` can be specified multiple times in a BLS entry file.
+/// Some keys like ```initrd``` can be specified multiple times in a BLS entry file.
 #[derive(Debug, PartialEq)]
 pub enum ValueSetPolicy {
     /// This replaces all existing values for the given key
@@ -55,6 +60,50 @@ pub enum ValueSetPolicy {
     InsertAt(usize),
 }
 
+#[derive(Debug, PartialEq)]
+pub enum BLSKey {
+    Title,
+    Version,
+    MachineId,
+    SortKey,
+    Linux,
+    Efi,
+    Initrd,
+    Options,
+    Devicetree,
+    DevicetreeOverlay,
+    Architecture,
+    GrubHotkey,
+    GrubUsers,
+    GrubClass,
+    GrubArg,
+}
+
+impl FromStr for BLSKey {
+    type Err = String;
+
+    fn from_str(key: &str) -> Result<Self, Self::Err> {
+        match key {
+            "linux" => Ok(BLSKey::Linux),
+            "title" => Ok(BLSKey::Title),
+            "version" => Ok(BLSKey::Version),
+            "machine_id" => Ok(BLSKey::MachineId),
+            "sort_key" => Ok(BLSKey::SortKey),
+            "efi" => Ok(BLSKey::Efi),
+            "initrd" => Ok(BLSKey::Initrd),
+            "options" => Ok(BLSKey::Options),
+            "devicetree" => Ok(BLSKey::Devicetree),
+            "devicetree_overlay" => Ok(BLSKey::DevicetreeOverlay),
+            "architecture" => Ok(BLSKey::Architecture),
+            "grub_hotkey" => Ok(BLSKey::GrubHotkey),
+            "grub_users" => Ok(BLSKey::GrubUsers),
+            "grub_class" => Ok(BLSKey::GrubClass),
+            "grub_arg" => Ok(BLSKey::GrubArg),
+            _ => Err(format!("Invalid key {}", key)),
+        }
+    }
+}
+
 /// BLSEntry represents the contents of a BLS entry file
 #[derive(Debug)]
 pub struct BLSEntry {
@@ -105,7 +154,7 @@ impl BLSEntry {
     /// order commands and comments are not preserved.
     pub fn parse(buffer: &str) -> Result<BLSEntry, String> {
         let mut entry = BLSEntry::new();
-        let mut has_kernel = false;
+        let mut has_linux = false;
 
         for line in buffer.lines() {
             let mut comment = None;
@@ -121,17 +170,17 @@ impl BLSEntry {
             // NOTE: For now we put all comment lines in the header
             if line.trim().contains(" ") {
                 let key_value: Vec<&str> = line.trim().splitn(2, " ").collect();
-                if key_value[0] == "linux" {
-                    has_kernel = true;
+
+                let key = BLSKey::from_str(key_value[0])?;
+                if key == BLSKey::Linux {
+                    has_linux = true;
                 }
-                if let Err(error) = entry.set(
-                    key_value[0],
+                entry.set(
+                    key,
                     String::from(key_value[1]),
                     comment,
                     ValueSetPolicy::Append,
-                ) {
-                    return Err(error);
-                }
+                );
             } else {
                 match comment {
                     Some(comment) => {
@@ -142,7 +191,7 @@ impl BLSEntry {
             }
         }
 
-        if has_kernel {
+        if has_linux {
             Ok(entry)
         } else {
             Err(String::from("No 'linux' command found."))
@@ -210,30 +259,22 @@ impl BLSEntry {
 
     /// Sets a value for a given key
     /// # Arguments
-    /// 
+    ///
     /// - ```key```: a &str representing the key to be set
     /// - ```value```: a String representing the value for the key
     /// - ```comment```: an optional String reprensenting an inline comment for the value
     /// - ```set_policy```: Some keys can hold multiple values, the ```ValueSetPolicy``` enum specifies the policy for keys that can be specified multiple times.
-    /// 
-    /// # Returns
-    /// 
-    /// Ok(()) if the key was succesfully set.
-    /// 
-    /// # Errors
     ///
-    /// Returns an Err(String) if an invalid key is set.
-    /// 
     /// # Panics
-    /// 
+    ///
     /// If ```ValueSetPolicy::InsertAt(usize)``` is used as ```set_policy``` it may cause a panic if the index is out of bound
     pub fn set(
         &mut self,
-        key: &str,
+        key: BLSKey,
         value: String,
         comment: Option<String>,
         set_policy: ValueSetPolicy,
-    ) -> Result<(), String> {
+    ) {
         fn value_generator(value: String, comment: Option<String>) -> BLSValue {
             match comment {
                 Some(comment) => BLSValue::ValueWithComment(value, comment),
@@ -254,38 +295,62 @@ impl BLSEntry {
         }
 
         match key {
-            "title" => self.title = Some(value_generator(value, comment)),
-            "version" => self.version = Some(value_generator(value, comment)),
-            "machine-id" => self.machine_id = Some(value_generator(value, comment)),
-            "sort-key" => self.sort_key = Some(value_generator(value, comment)),
-            "linux" => self.linux = value_generator(value, comment),
-            "efi" => self.efi = Some(value_generator(value, comment)),
-            "devicetree" => self.devicetree = Some(value_generator(value, comment)),
-            "devicetree-overlay" => self.devicetree_overlay = Some(value_generator(value, comment)),
-            "architecture" => self.architecture = Some(value_generator(value, comment)),
-            "grub_hotkey" => self.grub_hotkey = Some(value_generator(value, comment)),
-            "grub_users" => self.grub_users = Some(value_generator(value, comment)),
-            "grub_arg" => self.grub_arg = Some(value_generator(value, comment)),
+            BLSKey::Title => self.title = Some(value_generator(value, comment)),
+            BLSKey::Version => self.version = Some(value_generator(value, comment)),
+            BLSKey::MachineId => self.machine_id = Some(value_generator(value, comment)),
+            BLSKey::SortKey => self.sort_key = Some(value_generator(value, comment)),
+            BLSKey::Linux => self.linux = value_generator(value, comment),
+            BLSKey::Efi => self.efi = Some(value_generator(value, comment)),
+            BLSKey::Devicetree => self.devicetree = Some(value_generator(value, comment)),
+            BLSKey::DevicetreeOverlay => {
+                self.devicetree_overlay = Some(value_generator(value, comment))
+            }
+            BLSKey::Architecture => self.architecture = Some(value_generator(value, comment)),
+            BLSKey::GrubHotkey => self.grub_hotkey = Some(value_generator(value, comment)),
+            BLSKey::GrubUsers => self.grub_users = Some(value_generator(value, comment)),
+            BLSKey::GrubArg => self.grub_arg = Some(value_generator(value, comment)),
 
-            "initrd" => push_value(
+            BLSKey::Initrd => push_value(
                 &mut self.initrd,
                 value_generator(value, comment),
                 set_policy,
             ),
-            "options" => push_value(
+            BLSKey::Options => push_value(
                 &mut self.options,
                 value_generator(value, comment),
                 set_policy,
             ),
-            "grub_class" => push_value(
+            BLSKey::GrubClass => push_value(
                 &mut self.grub_class,
                 value_generator(value, comment),
                 set_policy,
             ),
-            _ => return Err(format!("Invalid key {}", key)),
         }
+    }
 
-        Ok(())
+    /// Clears a field in the BLSEntry
+    /// 
+    /// # Arguments
+    /// - key: The ```BLSKey``` to clear. Note that if ```BLSKey::linux``` is used it will set it to an empty string as this key is always supposed to be present. Sets None or empty vector otherwise.
+    pub fn clear(&mut self, key: BLSKey) {
+        match key {
+            BLSKey::Linux => self.linux = BLSValue::Value(String::from("")),
+            BLSKey::Title => self.title = None,
+            BLSKey::Version => self.version = None,
+            BLSKey::MachineId => self.machine_id = None,
+            BLSKey::SortKey => self.sort_key = None,
+            BLSKey::Efi => self.efi = None,
+            BLSKey::Devicetree => self.devicetree = None,
+            BLSKey::DevicetreeOverlay => self.devicetree_overlay = None,
+            BLSKey::Architecture => self.architecture = None,
+            BLSKey::GrubHotkey => self.grub_hotkey = None,
+            BLSKey::GrubUsers => self.grub_users = None,
+            BLSKey::GrubArg => self.grub_arg = None,
+
+            BLSKey::Initrd => self.initrd.clear(),
+            BLSKey::Options => self.options.clear(),
+            BLSKey::GrubClass => self.grub_class.clear(),
+        }
     }
 }
 
@@ -297,6 +362,7 @@ mod bls_tests {
     use alloc::vec;
 
     use super::BLSEntry;
+    use super::BLSKey;
     use super::BLSValue;
     use super::ValueSetPolicy;
 
@@ -360,9 +426,24 @@ mod bls_tests {
     fn set_value_policies() {
         // Append
         let mut entry = BLSEntry::new();
-        let _ = entry.set("options", String::from("foo"), None, ValueSetPolicy::Append);
-        let _ = entry.set("options", String::from("bar"), None, ValueSetPolicy::Append);
-        let _ = entry.set("options", String::from("baz"), None, ValueSetPolicy::Append);
+        let _ = entry.set(
+            BLSKey::Options,
+            String::from("foo"),
+            None,
+            ValueSetPolicy::Append,
+        );
+        let _ = entry.set(
+            BLSKey::Options,
+            String::from("bar"),
+            None,
+            ValueSetPolicy::Append,
+        );
+        let _ = entry.set(
+            BLSKey::Options,
+            String::from("baz"),
+            None,
+            ValueSetPolicy::Append,
+        );
 
         assert_eq!(
             entry.options,
@@ -375,7 +456,7 @@ mod bls_tests {
 
         // InsertAt
         let _ = entry.set(
-            "options",
+            BLSKey::Options,
             String::from("lol"),
             None,
             ValueSetPolicy::InsertAt(1),
@@ -392,7 +473,7 @@ mod bls_tests {
 
         // ReplaceAll
         let _ = entry.set(
-            "options",
+            BLSKey::Options,
             String::from("wtf"),
             None,
             ValueSetPolicy::ReplaceAll,
@@ -401,7 +482,7 @@ mod bls_tests {
 
         // Prepend
         let _ = entry.set(
-            "options",
+            BLSKey::Options,
             String::from("uwu"),
             None,
             ValueSetPolicy::Prepend,
@@ -413,5 +494,20 @@ mod bls_tests {
                 BLSValue::Value(String::from("wtf"))
             ]
         );
+
+        // Clear
+        entry.clear(BLSKey::Options);
+        assert_eq!(entry.options, vec![]);
+
+        entry.set(
+            BLSKey::Title,
+            String::from("foobar"),
+            None,
+            ValueSetPolicy::Append,
+        );
+        assert_eq!(entry.title, Some(BLSValue::Value(String::from("foobar"))));
+
+        entry.clear(BLSKey::Title);
+        assert_eq!(entry.title, None);
     }
 }
